from data.compression.compression_base import Compression
from typing.io import BinaryIO
from data.transmission.transmission_base import Transmission
import os


class NoSuchExtensionError(Exception):
    pass


class Manager:
    """
    Manages data, has functions such as save to file, load etc...
    """

    def __init__(self, compression: Compression, transmission: Transmission):
        self.trans = transmission
        self.comp = compression
        self._tmp_path = './.seclea/'
        self._tmp_file = 'tmp'
        self._tmp_file_comp = 'tmp_comp'
        if not os.path.exists(self._tmp_path):
            os.makedirs(self._tmp_path)
        pass

    def download_file_from_server(self, model_path, server_query_params):
        """

        """
        self.trans.query_params = server_query_params
        self.trans.load_file(model_path)


    def send_file(self, path: str):
        """

        """
        self.trans.send_file(path)
        pass

    def _save_to_tmp(self, model_encoded):
        open(self._tmp_path + self._tmp_file, 'wb').write(model_encoded)

    def save_object_to_path(self, object, path: str) -> str:
        """
        Saves and compresses an object to a path.
        @returns: str path of saved object.
        """
        self._save_to_tmp(object)
        obj_hash = hash(object)
        save_path = path + str(obj_hash) + self.comp.extension
        self.comp.compress(open(self._tmp_path + self._tmp_file, 'rb'),
                           open(save_path, 'wb'))
        return save_path

    def save_file_to_path(self, file: BinaryIO, path: str):
        path_out = os.path.join(path, file.name) + self.comp.extension
        self.comp.compress(read_stream=file, write_stream=open(path_out, 'wb'))
        return path_out
        pass

    def load_file_from_path(self, path) -> BinaryIO:
        """
        decompresses and loads file from path
        """
        try:
            # decompress the file first if compressed.
            new_path = Manager.remove_extension(path, self.comp.extension)
            self.comp.decompress(read_stream=open(path, 'rb'), write_stream=open(new_path, 'wb'))
        except NoSuchExtensionError:
            # file is uncompressed, just load it.
            new_path = path
        return open(new_path, 'rb')

    @staticmethod
    def remove_extension(path, expected_ext):
        """
        if the extension can be removed it will be otherwise the path is returned as is.
        """
        l = path.split('.')
        if l[-1] != expected_ext:
            raise NoSuchExtensionError
        relevant = l[:-1]
        return '.'.join(relevant)


print(Manager.remove_extension('some_path/some-file.file/.git/some_file.py'))
